@@ -51,6 +51,7 @@ my $cr_dir="";
 my $debug='';
 my $debug_image_dir='';
 my $debug_image='';
+my $debug_pixels=0;
 my $progress=0;
 my $progress_id=0;
 my $scans_list;
@@ -71,6 +72,7 @@ GetOptions("data=s"=>\$data_dir,
 	   "prop=s"=>\$prop,
 	   "bw-threshold=s"=>\$bw_threshold,
 	   "debug=s"=>\$debug,
+	   "debug-pixels!"=>\$debug_pixels,
 	   "progression=s"=>\$progress,
 	   "progression-id=s"=>\$progress_id,
 	   "liste-fichiers=s"=>\$scans_list,
@@ -115,11 +117,13 @@ if($scans_list && open(LISTE,$scans_list)) {
 exit(0) if($#scans <0);
 
 sub error {
-    my ($e,$silent)=shift;
-    if($debug_image &&
-       $process->mode() eq 'opencv') {
+    my ($process,$e,$silent)=@_;
+    if($process) {
+      if($debug_image &&
+	 $process->mode() eq 'opencv') {
 	$process->commande("output ".$debug_image);
 	$process->ferme_commande;
+      }
     }
     if($silent) {
 	debug $e;
@@ -186,27 +190,21 @@ sub get_layout_data {
     =$layout->dims($student,$page);
   $r->{'frame'}=AMC::Boite::new_complete($layout->all_marks($student,$page));
 
-  my $sth=$layout->statement('digitInfo');
-  $sth->execute($student,$page);
-  while(my $c=$sth->fetchrow_hashref) {
+  for my $c ($layout->type_info('digit',$student,$page)) {
     my $k=code_cb($c->{'numberid'},$c->{'digitid'});
     $r->{'boxes'}->{$k}=AMC::Boite::new_MN(map { $c->{$_} }
 					   (qw/xmin ymin xmax ymax/));
   }
 
   if($all) {
-    $sth=$layout->statement('boxInfo');
-    $sth->execute($student,$page);
-    while($c=$sth->fetchrow_hashref) {
+    for my $c ($layout->type_info('box',$student,$page)) {
       $r->{'boxes'}->{$c->{'question'}.".".$c->{'answer'}}=
 	AMC::Boite::new_MN(map { $c->{$_} }
 			   (qw/xmin ymin xmax ymax/));
       $r->{'flags'}->{$c->{'question'}.".".$c->{'answer'}}=
 	$c->{'flags'};
     }
-    $sth=$layout->statement('namefieldInfo');
-    $sth->execute($student,$page);
-    while($c=$sth->fetchrow_hashref) {
+    for my $c ($layout->type_info('namefield',$student,$page)) {
       $r->{'boxes'}->{'namefield'}=
 	AMC::Boite::new_MN(map { $c->{$_} }
 			   (qw/xmin ymin xmax ymax/));
@@ -226,7 +224,7 @@ $layout->begin_read_transaction('cRLY');
 
 if($layout->pages_count()==0) {
   $layout->end_transaction('cRLY');
-  error("No layout");
+  error('',"No layout");
 }
 debug "".$layout->pages_count()." layouts\n";
 
@@ -275,6 +273,14 @@ sub marks_fit {
   $ld->{'transf'}=$cale;
 }
 
+sub get_shape {
+  my ($flags)=@_;
+  if($flags & BOX_FLAGS_SHAPE_OVAL) {
+    return('oval');
+  }
+  return('square');
+}
+
 ##################################################
 # Reads darkness of a particular box
 
@@ -290,13 +296,27 @@ sub measure_box {
 	}
     }
 
-    $ld->{'boxes.scan'}->{$k}=$ld->{'boxes'}->{$k}->clone;
-    $ld->{'boxes.scan'}->{$k}->transforme($ld->{'transf'});
+    if($process->mode() eq 'opencv' &&
+       !($ld->{'flags'}->{$k} & BOX_FLAGS_DONTSCAN)) {
+      $ld->{'boxes.scan'}->{$k}=AMC::Boite::new();
+    } else {
+      $ld->{'boxes.scan'}->{$k}=$ld->{'boxes'}->{$k}->clone;
+      $ld->{'boxes.scan'}->{$k}->transforme($ld->{'transf'});
+    }
 
     if(!($ld->{'flags'}->{$k} & BOX_FLAGS_DONTSCAN)) {
-      for($process->commande($ld->{'boxes.scan'}->{$k}
-			     ->commande_mesure($prop))) {
-
+      my $pc;
+      if($process->mode() eq 'opencv') {
+	$pc=$ld->{'boxes'}->{$k}
+	  ->commande_mesure0($prop,get_shape($ld->{'flags'}->{$k}));
+      } else {
+	$pc=$ld->{'boxes.scan'}->{$k}
+	  ->commande_mesure($prop);
+      }
+      for($process->commande($pc)) {
+	if(/^TCORNER\s+(-?[0-9\.]+),(-?[0-9\.]+)$/) {
+	  $ld->{'boxes.scan'}->{$k}->def_point_suivant($1,$2);
+	}
 	if(/^COIN\s+(-?[0-9\.]+),(-?[0-9\.]+)$/) {
 	  $ld->{'corners.test'}->{$k}->def_point_suivant($1,$2);
 	}
@@ -427,7 +447,7 @@ sub one_scan {
     }
     close(CMD);
 
-    error("Unknown size") if(!$tiff_x);
+    error('',"Unknown size") if(!$tiff_x);
 
     debug "IMAGE = $scan ($tiff_x x $tiff_y)\n";
   }
@@ -451,6 +471,7 @@ sub one_scan {
 
     push @args,'-P' if($debug_image);
     push @args,'-r' if($ignore_red);
+    push @args,'-k' if($debug_pixels);
 
     $process->set('args',\@args);
 
@@ -504,7 +525,8 @@ sub one_scan {
     @okbox=grep { $_->bonne_etendue($taille_min,$taille_max) } @box;
 
     if($#okbox < 3) {
-	error("Only ".(1+$#okbox)." signs detected / needs at least 4");
+	error($process,
+	      "Only ".(1+$#okbox)." signs detected / needs at least 4");
     }
 
     @okbox=AMC::Boite::extremes(@okbox);
@@ -558,7 +580,7 @@ sub one_scan {
     $capture->failed($sf);
     $capture->end_transaction('CFLD');
 
-    error(sprintf("No layout for ID +%d/%d/%d+",@epc)) ;
+    error($process,sprintf("No layout for ID +%d/%d/%d+",@epc)) ;
   }
 
   if($process->mode() eq 'opencv') {
@@ -605,22 +627,11 @@ sub one_scan {
     }
   }
 
-  my $zoom_dir;
-  my $zoom_subdir;
+  my $zoom_dir = tempdir( DIR=>tmpdir(),
+			  CLEANUP => (!get_debug()) );
 
-  if($cr_dir) {
-
-    $zoom_subdir=pageids_string(@spc,'path'=>1);
-    $zoom_dir="$cr_dir/zooms/".$zoom_subdir;
-
-    # clear old analysis results files
-
-    clear_old('analysis result',
-	      $zoom_dir);
-
-    if($process->mode() eq 'opencv') {
-      $process->commande("zooms $zoom_dir");
-    }
+  if($process->mode() eq 'opencv') {
+    $process->commande("zooms $zoom_dir");
   }
 
   ##########################################
@@ -635,7 +646,7 @@ sub one_scan {
     $process->commande("annote ".pageids_string(@spc));
   }
 
-  error("End of diagnostic",1) if($debug_image);
+  error($process,"End of diagnostic",1) if($debug_image);
 
   ##########################################
   # Creates layout image report
@@ -766,18 +777,19 @@ sub one_scan {
       $zoneid=$capture->get_zoneid(@spc,ZONE_DIGIT,$n,$i,1);
     } elsif($k eq 'namefield') {
       $zoneid=$capture->get_zoneid(@spc,ZONE_NAME,0,0,1);
-      $capture->statement('setZoneAuto')
-	->execute(-1,-1,$nom_file,$zoneid);
+      $capture->set_zone_auto_id($zoneid,-1,-1,$nom_file,undef);
     }
 
     if($zoneid) {
       if($k ne 'namefield') {
 	if($ld->{'flags'}->{$k} & BOX_FLAGS_DONTSCAN) {
 	  debug "Box $k is DONT_SCAN";
+	  $capture->set_zone_auto_id($zoneid,1,0,undef,undef);
 	} elsif($ld->{'darkness.data'}->{$k}) {
-	  $capture->statement('setZoneAuto')
-	    ->execute(@{$ld->{'darkness.data'}->{$k}},
-		      $zoom_subdir."/".$ld->{'zoom.file'}->{$k},$zoneid);
+	  $capture->set_zone_auto_id($zoneid,
+			   @{$ld->{'darkness.data'}->{$k}},
+			   undef,
+			   file_content($zoom_dir."/".$ld->{'zoom.file'}->{$k}));
 	} else {
 	  debug "No darkness data for box $k";
 	}
