@@ -23,6 +23,8 @@ use Getopt::Long;
 use Gtk2;
 use Cairo;
 
+use List::Util qw(min max sum);
+
 use AMC::Basic;
 use AMC::Exec;
 use AMC::Gui::Avancement;
@@ -30,6 +32,8 @@ use AMC::NamesFile;
 use AMC::Data;
 use AMC::DataModule::capture qw/:zone :position/;
 use AMC::DataModule::layout qw/:flags/;
+use AMC::Substitute;
+
 use encoding 'utf8';
 
 my $cr_dir="";
@@ -62,6 +66,7 @@ my $pointsize_rel=60;
 my $chiffres_significatifs=4;
 
 my $verdict='TOTAL : %S/%M => %s/%m';
+my $verdict_question_cancelled='"X"';
 my $verdict_question='';
 
 my $font_name='FreeSans';
@@ -104,6 +109,7 @@ GetOptions("cr=s"=>\$cr_dir,
 	   "ch-sign=s"=>\$chiffres_significatifs,
 	   "verdict=s"=>\$verdict,
 	   "verdict-question=s"=>\$verdict_question,
+	   "verdict-question-cancelled=s"=>\$verdict_question_cancelled,
 	   "fich-noms=s"=>\$fich_noms,
 	   "noms-encodage=s"=>\$noms_encodage,
 	   "csv-build-name=s"=>\$csv_build_name,
@@ -150,14 +156,6 @@ sub color_rgb {
     return($col->red/65535,$col->green/65535,$col->blue/65535);
 }
 
-sub format_note {
-    my $x=shift;
-    if($chiffres_significatifs>0) {
-	$x=sprintf("%.*g",$chiffres_significatifs,$x);
-    }
-    return($x);
-}
-
 my $avance=AMC::Gui::Avancement::new($progress,'id'=>$progress_id);
 
 my $data=AMC::Data->new($data_dir);
@@ -167,7 +165,6 @@ my $assoc=$data->module('association');
 my $layout=$data->module('layout');
 
 $seuil=$scoring->variable_transaction('darkness_threshold');
-$lk=$assoc->variable_transaction('key_in_list');
 
 #################################
 
@@ -243,6 +240,12 @@ if($id_file) {
 
 }
 
+my $subst=AMC::Substitute::new('names'=>$noms,
+			       'scoring'=>$scoring,
+			       'assoc'=>$assoc,
+			       'name'=>'',
+			       'chsign'=>$chiffres_significatifs,
+			       );
 
 print "* Annotation\n";
 
@@ -328,40 +331,10 @@ print "* Annotation\n";
 
     $capture->begin_read_transaction('xSTD');
 
-    my $student_mark=$scoring->student_global(@spc[0,2]);
-
-    if(!$student_mark) {
-      print "*** no marks for copy ".studentids_string(@spc[0,2])." ***\n";
-      debug "No marks found ! Copy=".studentids_string(@spc[0,2]);
-      next PAGE;
-    }
-
     # print global mark and name on the page
 
     if($p->{'page'}==1 || $capture->zones_count(@spc,ZONE_NAME)) {
-      my $text=$verdict;
-
-      $text =~ s/\%[S]/format_note($student_mark->{'total'})/ge;
-      $text =~ s/\%[M]/format_note($student_mark->{'max'})/ge;
-      $text =~ s/\%[s]/format_note($student_mark->{'mark'})/ge;
-      $text =~ s/\%[m]/format_note($scoring->variable('mark_max'))/ge;
-
-      if($assoc && $noms) {
-	my $i=$assoc->get_real(@spc[0,2]);
-	my $n;
-
-	debug "Association -> ID=$i";
-
-	if($i) {
-	  debug "Name found";
-	  ($n)=$noms->data($lk,$i);
-	  if($n) {
-	    $text=$noms->substitute($n,$text,'prefix'=>'%');
-	  }
-	}
-      } else {
-	debug "No association/names";
-      }
+      my $text=$subst->substitute($verdict,@spc[0,2]);
 
       $lay->set_text($text);
       $context->set_source_rgb(color_rgb('red'));
@@ -425,12 +398,8 @@ print "* Annotation\n";
 
       $question{$q}={} if(!$question{$q});
       my @mil=milieu_cercle($b->{'zoneid'});
-      $question{$q}->{'n'}++;
-      $question{$q}->{'x'}=$mil[0]
-	if((!$question{$q}->{'x'}) || ($mil[0]<$question{$q}->{'x'}));
-      $question{$q}->{'xmax'}=$mil[0]
-	if((!$question{$q}->{'xmax'}) || ($mil[0]>$question{$q}->{'xmax'}));
-      $question{$q}->{'y'}+=$mil[1];
+      push @{$question{$q}->{'x'}},$mil[0];
+      push @{$question{$q}->{'y'}},$mil[1];
     }
 
     #########################################
@@ -443,12 +412,19 @@ print "* Annotation\n";
 
 	my $result=$scoring->question_result(@spc[0,2],$q);
 
-	my $text=$verdict_question;
+	my $text;
+
+	if($result->{'why'} =~ /c/i) {
+	  $text=$verdict_question_cancelled;
+	} else {
+	  $text=$verdict_question;
+	}
 
 	$text =~ s/\%[S]/$result->{'score'}/g;
 	$text =~ s/\%[M]/$result->{'max'}/g;
-	$text =~ s/\%[s]/format_note($result->{'score'})/ge;
-	$text =~ s/\%[m]/format_note($result->{'max'})/ge;
+	$text =~ s/\%[W]/$result->{'why'}/g;
+	$text =~ s/\%[s]/$subst->format_note($result->{'score'})/ge;
+	$text =~ s/\%[m]/$subst->format_note($result->{'max'})/ge;
 
 	my $te=eval($text);
 	if($@) {
@@ -460,26 +436,52 @@ print "* Annotation\n";
 
 	$lay->set_text($text);
 	my ($tx,$ty)=$lay->get_pixel_size;
+
+	# mean of the y coordinate of all boxes
+	my $y=sum(@{$question{$q}->{'y'}})/(1+$#{$question{$q}->{'y'}})-$ty/2;
+
 	if($position eq 'marge') {
+	  # scores written in one margin
 	  if($rtl) {
 	    $x=$page_width-$ecart_marge*$text_x-$tx;
 	  } else {
 	    $x=$ecart_marge*$text_x;
 	  }
 	} elsif($position eq 'case') {
+	  # scores written at the left of the boxes
 	  if($rtl) {
-	    $x=$question{$q}->{'xmax'} + $ecart*$text_x ;
+	    $x=max(@{$question{$q}->{'x'}}) + $ecart*$text_x ;
 	  } else {
-	    $x=$question{$q}->{'x'} - $ecart*$text_x - $tx;
+	    $x=min(@{$question{$q}->{'x'}}) - $ecart*$text_x - $tx;
 	  }
+	} elsif($position eq 'marges') {
+	  # scores written in one of the margins (left or right),
+	  # depending on the position of the boxes. This mode is often
+	  # used when the subject is in a 2-column layout.
+
+	  # fist extract the y coordinates of the boxes in the left column
+	  my $left=1;
+	  my @y=map { $question{$q}->{'y'}->[$_] } grep { $rtl xor ( $question{$q}->{'x'}->[$_] <= $page_width/2 ) } (0..$#{$question{$q}->{'x'}} );
+	  if(!@y) {
+	    # if empty, use the right column
+	    $left=0;
+	    @y=map { $question{$q}->{'y'}->[$_] } grep { $rtl xor ( $question{$q}->{'x'}->[$_] > $page_width/2 ) } (0..$#{$question{$q}->{'x'}} );
+	  }
+
+	  # set the x-position to the right margin
+	  if($left xor $rtl) {
+	    $x=$ecart_marge*$text_x;
+	  } else {
+	    $x=$page_width-$ecart_marge*$text_x-$tx;
+	  }
+	  # set the y-position to the mean of y coordinates of the
+	  # boxes in the corresponding column
+	  $y=sum(@y)/(1+$#y)-$ty/2;
 	} else {
 	  debug "Annotation : position invalide : $position";
-	  $x=$text_x;
+	  $x=$ecart_marge*$text_x;
 	}
 
-	# moyenne des y des cases de la question
-	my $y=$question{$q}->{'y'}/$question{$q}->{'n'}-$ty/2;
-
 	$context->set_source_rgb(color_rgb('red'));
 	$context->move_to($x,$y);
 	Pango::Cairo::show_layout($context,$lay);
@@ -496,8 +498,26 @@ print "* Annotation\n";
 
     debug "Saving annotated scan to $cr_dir/corrections/jpg/$out_file";
 
+    my @args=();
+
+    if($qualite_jpg) {
+      if($qualite_jpg =~ /^[0-9]+$/) {
+	push @args,"-quality",$qualite_jpg;
+      } else {
+	debug "WARNING: non-numeric --qualite argument, ignored ($qualite_jpg)";
+      }
+    }
+
+    if($taille_max) {
+      if($taille_max =~ /^[0-9]*x?[0-9]*$/) {
+	push @args,"-geometry",$taille_max;
+      } else {
+	debug "WARNING: malformed --taille-max argument, ignored ($taille_max)";
+      }
+    }
+
     open(CONV,"|-",magick_module("convert"),"png:-",
-	 "-quality",$qualite_jpg,"-geometry",$taille_max,
+	 @args,
 	 "$cr_dir/corrections/jpg/$out_file");
     $surface->write_to_png_stream(
 				  sub {
