@@ -30,10 +30,12 @@ my $liste_enc='utf-8';
 my $csv_build_name='';
 my $data_dir='';
 my $debug='';
+my $preassoc='';
 
 @ARGV=unpack_args(@ARGV);
 
 GetOptions("notes-id=s"=>\$notes_id,
+	   "pre-association!"=>\$preassoc,
 	   "liste=s"=>\$liste_file,
 	   "liste-key=s"=>\$liste_key,
 	   "csv-build-name=s"=>\$csv_build_name,
@@ -44,7 +46,7 @@ GetOptions("notes-id=s"=>\$notes_id,
 
 set_debug($debug);
 
-die "Needs notes-id" if(!$notes_id);
+die "Needs notes-id" if(!$notes_id && !$preassoc);
 die "Needs liste-key" if(!$liste_key);
 die "Needs liste_file" if(! -s $liste_file);
 die "Needs data_dir" if(!-d $data_dir);
@@ -53,9 +55,22 @@ my $data=AMC::Data->new($data_dir);
 my $scoring=$data->module('scoring');
 my $assoc=$data->module('association');
 my $capture=$data->module('capture');
+my $layout;
+
+$layout=$data->module('layout')
+  if($preassoc);
 
 debug "Automatic association $liste_file [$liste_enc] / $liste_key";
 
+# function that "cleans" IDs, removing leading zeros (so that 0001234
+# will be the same as 1234)
+
+sub clean_id {
+  my ($i)=@_;
+  $i =~ s/^0+//;
+  return($i);
+}
+
 # First read from the students list the possible values for the
 # primary key to be found there (from column named $liste_key).
 
@@ -65,10 +80,11 @@ my $liste_e=AMC::NamesFile::new($liste_file,
 
 my %bon_code;
 for my $ii (0..($liste_e->taille()-1)) {
-    $bon_code{$liste_e->data_n($ii,$liste_key)}=1;
+  my $id=$liste_e->data_n($ii,$liste_key);
+  $bon_code{clean_id($id)}=$id;
 }
 
-debug "Student list keys: ".join(',',keys %bon_code);
+debug "Cleaned student list keys: ".join(',',keys %bon_code);
 
 # Open association database and clear old automatic association
 
@@ -80,23 +96,29 @@ $assoc->clear_auto;
 
 # Loop on all codes that can be read on the scans.
 
-my $sth=$scoring->statement('codesCounts');
-$sth->execute($notes_id);
+my $sth=$scoring->statement($preassoc ? 'preAssocCounts' : 'codesCounts');
+if($preassoc) {
+  $sth->execute();
+} else {
+  $sth->execute($notes_id);
+}
 while(my $v=$sth->fetchrow_hashref) {
   if($v->{'nb'}==1) {
     # nb is the number of scans on which the same code value has been
     # read. If nb=1, this is OK: we can process association...
-    if($bon_code{$v->{'value'}}) {
-      # ... unless this value is NOT in the students list!
-      debug "Association OK for code value $v->{'value'}";
-      $assoc->set_auto(map { $v->{$_} } (qw/student copy value/));
-    } else {
+
+    my $id_in_list=$bon_code{clean_id($v->{'value'})};
+    if(defined($id_in_list)) {
       # Association OK
+      debug "Association OK for code value $v->{'value'} ($id_in_list)";
+      $assoc->set_auto((map { $v->{$_} } (qw/student copy/)),$id_in_list);
+    } else {
+      # ... unless this value is NOT in the students list!
       debug "Code value $v->{'value'} not found in students list: ignoring";
     }
   } else {
     # Code value found on several sheets: do nothing, wait for the
-    # used to make a manual association for these sheets.
+    # user to make a manual association for these sheets.
     debug "Incorrect association for code value \"".$v->{'value'}."\": $v->{'nb'} instances";
   }
 }
