@@ -1,6 +1,6 @@
 #! /usr/bin/perl
 #
-# Copyright (C) 2011-2012 Alexis Bienvenue <paamc@passoire.fr>
+# Copyright (C) 2011-2013 Alexis Bienvenue <paamc@passoire.fr>
 #
 # This file is part of Auto-Multiple-Choice
 #
@@ -89,6 +89,7 @@ sub ajoute {
 
 my @pages=();
 my @flags=();
+my @pre_assoc=();
 my $cases;
 my $page_number=0;
 my $i='';
@@ -111,6 +112,8 @@ sub add_flag {
   }
 }
 
+debug "Reading $src...";
+
 open(SRC,$src) or die "Unable to open $src : $!";
 while(<SRC>) {
     if(/\\page{([^\}]+)}{([^\}]+)}{([^\}]+)}/) {
@@ -123,14 +126,25 @@ while(<SRC>) {
 		     -dim_x=>read_inches($dx),-dim_y=>read_inches($dy),
 		     -cases=>$cases};
     }
-    if(/\\tracepos\{([^\}]+)\}\{([^\}]*)\}\{([^\}]*)\}/) {
+    if(/\\tracepos\{([^\}]+)\}\{([^\}]*)\}\{([^\}]*)\}(?:\{([^\}]*)\})?/) {
 	$i=$1;
 	my $x=read_inches($2);
 	my $y=read_inches($3);
+	my $shape=$4;
 	$i =~ s/^[0-9]+\/[0-9]+://;
-	$cases->{$i}={'bx'=>[],'by'=>[],'flags'=>0} if(!$cases->{$i});
+	$cases->{$i}={'bx'=>[],'by'=>[],'flags'=>0,'shape'=>''}
+	  if(!$cases->{$i});
 	ajoute($cases->{$i}->{'bx'},$x);
 	ajoute($cases->{$i}->{'by'},$y);
+	if($cases->{$i}->{'shape'} &&
+	   $cases->{$i}->{'shape'} ne $shape) {
+	  debug "WARNING: different shapes for a single box ($i)";
+	} else {
+	  $cases->{$i}->{'shape'}=$shape;
+	  if($shape eq 'oval') {
+	    $cases->{$i}->{'flags'} |= BOX_FLAGS_SHAPE_OVAL;
+	  }
+	}
     }
     if(/\\dontscan\{(.*)\}/) {
       add_flag($1,BOX_FLAGS_DONTSCAN);
@@ -138,6 +152,9 @@ while(<SRC>) {
     if(/\\dontannotate\{(.*)\}/) {
       add_flag($1,BOX_FLAGS_DONTANNOTATE);
     }
+    if(/\\association\{([0-9]+)\}\{(.*)\}/) {
+      push @pre_assoc,[$1,$2];
+    }
 }
 close(SRC);
 
@@ -158,7 +175,15 @@ $layout->begin_transaction('MeTe');
 $layout->clear_all;
 $capture->variable('annotate_source_change',time());
 
-for my $p (@pages) {
+debug "Pre-association...";
+
+for my $pa (@pre_assoc) {
+  $layout->new_association(@$pa);
+}
+
+debug "Writing to database...";
+
+PAGE: for my $p (@pages) {
 
     my $diametre_marque=0;
     my $dmn=0;
@@ -176,11 +201,8 @@ for my $p (@pages) {
 	  }
       }
   }
-    $diametre_marque/=$dmn;
 
-    for my $pos ('HG','HD','BD','BG') {
-	die "Needs position$pos from page $p->{-id}" if(!$p->{-cases}->{'position'.$pos});
-    }
+    $diametre_marque/=$dmn if($dmn);
 
     my @epc=get_epc($p->{-id});
     my @ep=@epc[0,1];
@@ -192,6 +214,12 @@ for my $p (@pages) {
 	$diametre_marque,
 	$layout->source_id($src,$timestamp));
 
+    next PAGE if(!$dmn);
+
+    for my $pos ('HG','HD','BD','BG') {
+	die "Needs position$pos from page $p->{-id}" if(!$p->{-cases}->{'position'.$pos});
+    }
+
     my $c=$p->{-cases};
 
     my $nc=0;
@@ -216,17 +244,21 @@ for my $p (@pages) {
 	my ($name,$q,$a)=($1,$2,$3);
 	$layout->question_name($q,$name);
 	$layout->statement('NEWBox')
-	  ->execute(@ep,$q,$a,bbox($c->{$k}),0);
+	  ->execute(@ep,$q,$a,bbox($c->{$k}),$c->{$k}->{'flags'});
       }
     }
 
     $avance->progres($delta);
 }
 
+debug "Flagging questions...";
+
 for my $f (@flags) {
   $layout->add_question_flag($f->{'student'},$f->{'question'},$f->{'flags'});
 }
 
+debug "Ending transaction...";
+
 $layout->end_transaction('MeTe');
 
 $avance->fin();
