@@ -165,6 +165,9 @@ sub define_statements {
 			   ."(SELECT student,copy FROM $t_page"
 			   ." EXCEPT SELECT student,copy FROM $at"
 			   ." WHERE manual IS NOT NULL OR auto IS NOT NULL)"},
+     'deleteAssociations'=>{'sql'=>"DELETE FROM $at"
+			    ." WHERE student=? AND copy=?"},
+     'list'=>{'sql'=>"SELECT * FROM $at ORDER BY student,copy"},
     };
 }
 
@@ -315,4 +318,21 @@ sub missing_count {
   return($self->sql_single($self->statement('assocMissingCount')));
 }
 
+# delete_association_data($student,$copy) deletes all association data
+# for this answer sheet.
+
+sub delete_association_data {
+  my ($self,$student,$copy)=@_;
+  for my $part (qw/Associations/) {
+    $self->statement('delete'.$part)->execute($student,$copy);
+  }
+}
+
+# list() returns all association data
+
+sub list {
+  my ($self)=@_;
+  return($self->dbh->selectall_arrayref($self->statement('list'),{Slice=>{}}));
+}
+
 1;
