@@ -1,6 +1,6 @@
 # -*- perl -*-
 #
-# Copyright (C) 2011-2012 Alexis Bienvenue <paamc@passoire.fr>
+# Copyright (C) 2011-2013 Alexis Bienvenue <paamc@passoire.fr>
 #
 # This file is part of Auto-Multiple-Choice
 #
@@ -112,16 +112,24 @@ package AMC::DataModule::layout;
 # * question is the question ID (see explanation in layout_box)
 #
 # * name is the question identifier from the LaTeX file
+#
+# layout_association contains the pre-association data
+#
+# * student is the student sheet number
+#
+# * id is the association value (student id from the students list)
+#   for the corresponding student sheet
 
 use Exporter qw(import);
 
 use constant {
   BOX_FLAGS_DONTSCAN => 0x1,
   BOX_FLAGS_DONTANNOTATE => 0x2,
+  BOX_FLAGS_SHAPE_OVAL => 0x10,
 };
 
-our @EXPORT_OK = qw(BOX_FLAGS_DONTSCAN BOX_FLAGS_DONTANNOTATE);
-our %EXPORT_TAGS = ( 'flags' => [ qw/BOX_FLAGS_DONTSCAN BOX_FLAGS_DONTANNOTATE/ ],
+our @EXPORT_OK = qw(BOX_FLAGS_DONTSCAN BOX_FLAGS_DONTANNOTATE BOX_FLAGS_SHAPE_OVAL);
+our %EXPORT_TAGS = ( 'flags' => [ qw/BOX_FLAGS_DONTSCAN BOX_FLAGS_DONTANNOTATE BOX_FLAGS_SHAPE_OVAL/ ],
 		     );
 
 use AMC::Basic;
@@ -131,40 +139,80 @@ use XML::Simple;
 @ISA=("AMC::DataModule");
 
 sub version_current {
-  return(2);
+  return(4);
 }
 
 sub version_upgrade {
     my ($self,$old_version)=@_;
     if($old_version==0) {
 
-	# Upgrading from version 0 (empty database) to version 2 :
+	# Upgrading from version 0 (empty database) to version 4 :
 	# creates all the tables.
 
 	debug "Creating layout tables...";
 	$self->sql_do("CREATE TABLE IF NOT EXISTS ".$self->table("page")
 		      ." (student INTEGER, page INTEGER, checksum INTEGER, sourceid INTEGER, subjectpage INTEGER, dpi REAL, width REAL, height REAL, markdiameter REAL, PRIMARY KEY (student,page))");
 	$self->sql_do("CREATE TABLE IF NOT EXISTS ".$self->table("mark")
-		      ." (student INTEGER, page INTEGER, corner INTEGER, x REAL, y REAL)");
+		      ." (student INTEGER, page INTEGER, corner INTEGER, x REAL, y REAL, PRIMARY KEY (student,page,corner))");
 	$self->sql_do("CREATE TABLE IF NOT EXISTS ".$self->table("namefield")
 		      ." (student INTEGER, page INTEGER, xmin REAL, xmax REAL, ymin REAL, ymax REAL)");
+	$self->sql_do("CREATE INDEX ".$self->index("index_namefield")." ON "
+		      .$self->table("namefield","self")." (student,page)");
 	$self->sql_do("CREATE TABLE IF NOT EXISTS ".$self->table("box")
-		      ." (student INTEGER, page INTEGER, question INTEGER, answer INTEGER, xmin REAL, xmax REAL, ymin REAL, ymax REAL, flags INTEGER DEFAULT 0)");
+		      ." (student INTEGER, page INTEGER, question INTEGER, answer INTEGER, xmin REAL, xmax REAL, ymin REAL, ymax REAL, flags INTEGER DEFAULT 0, PRIMARY KEY (student,question,answer))");
+	$self->sql_do("CREATE INDEX ".$self->index("index_box_studentpage")." ON "
+		      .$self->table("box","self")." (student,page)");
 	$self->sql_do("CREATE TABLE IF NOT EXISTS ".$self->table("digit")
-		      ." (student INTEGER, page INTEGER, numberid INTEGER, digitid INTEGER, xmin REAL, xmax REAL, ymin REAL, ymax REAL)");
+		      ." (student INTEGER, page INTEGER, numberid INTEGER, digitid INTEGER, xmin REAL, xmax REAL, ymin REAL, ymax REAL, PRIMARY KEY(student,page,numberid,digitid))");
 	$self->sql_do("CREATE TABLE IF NOT EXISTS ".$self->table("source")
 		      ." (sourceid INTEGER PRIMARY KEY, src TEXT, timestamp INTEGER)");
 	$self->sql_do("CREATE TABLE IF NOT EXISTS ".$self->table("question")
 		      ." (question INTEGER PRIMARY KEY, name TEXT)");
+	$self->sql_do("CREATE TABLE IF NOT EXISTS ".$self->table("association")
+		      ." (student INTEGER PRIMARY KEY, id TEXT)");
 	$self->populate_from_xml;
 
-	return(2);
+	return(4);
     }
     if($old_version==1) {
       $self->sql_do("ALTER TABLE ".$self->table("box")
 		   ." ADD COLUMN flags DEFAULT 0");
       return(2);
     }
+    if($old_version==2) {
+      $self->sql_do("CREATE TABLE IF NOT EXISTS ".$self->table("association")
+		    ." (student INTEGER, id TEXT)");
+      return(3);
+    }
+    if($old_version==3) {
+      $self->progression('begin',__("Building layout database indexes..."));
+      # replaces missing PRIMARY KEYS with INDEXs
+      $self->sql_do("CREATE UNIQUE INDEX IF NOT EXISTS "
+		    .$self->index("index_box")." ON "
+		    .$self->table("box","self")." (student,question,answer)");
+      $self->progression('fraction',1/6);
+      $self->sql_do("CREATE INDEX IF NOT EXISTS "
+		    .$self->index("index_box_studentpage")." ON "
+		    .$self->table("box","self")." (student,page)");
+      $self->progression('fraction',1/6);
+      $self->sql_do("CREATE INDEX IF NOT EXISTS "
+		    .$self->index("index_namefield")." ON "
+		    .$self->table("namefield","self")." (student,page)");
+      $self->progression('fraction',1/6);
+      $self->sql_do("CREATE UNIQUE INDEX IF NOT EXISTS "
+		    .$self->index("index_digit")." ON "
+		    .$self->table("digit","self")." (student,page,numberid,digitid)");
+      $self->progression('fraction',1/6);
+      $self->sql_do("CREATE UNIQUE INDEX IF NOT EXISTS "
+		    .$self->index("index_mark")." ON "
+		    .$self->table("mark","self")." (student,page,corner)");
+      $self->progression('fraction',1/6);
+      $self->sql_do("CREATE INDEX IF NOT EXISTS "
+		    .$self->index("index_association")." ON "
+		    .$self->table("association","self")." (student)");
+      $self->progression('end');
+      return(4);
+    }
     return('');
 }
 
@@ -275,6 +323,8 @@ sub define_statements {
 			." (student,page,xmin,xmax,ymin,ymax) VALUES (?,?,?,?,?,?)"},
        'NEWQuestion'=>{'sql'=>"INSERT INTO ".$self->table("question")
 		       ." (question,name) VALUES (?,?)"},
+       'NEWAssociation'=>{'sql'=>"INSERT INTO ".$self->table("association")
+			  ." (student,id) VALUES (?,?)"},
        'IDS'=>{'sql'=>"SELECT student || ',' || page FROM ".$self->table("page")
 	       ." ORDER BY student,page"},
        'FULLIDS'=>{'sql'=>"SELECT '+' || student || '/' || page || '/' || checksum || '+' FROM "
@@ -329,9 +379,19 @@ sub define_statements {
 			   ." WHERE student=? AND page=?"},
        'students'=>{'sql'=>"SELECT student FROM ".$self->table("page")
 		    ." GROUP BY student"},
-       'subjectpageForStudent'=>{'sql'=>"SELECT subjectpage FROM ".$self->table("page")
-				 ." WHERE student=? ORDER BY page"},
+       'subjectpageForStudent'=>
+       {'sql'=>"SELECT MIN(subjectpage),MAX(subjectpage) FROM ".$self->table("page")
+	." WHERE student=?"},
+       'subjectpageForStudentA'=>
+       {'sql'=>"SELECT MIN(p.subjectpage),MAX(p.subjectpage) FROM "
+	.$self->table("page")." AS p"
+	." ,( SELECT student,page FROM ".$self->table("box")
+	."    UNION"
+	."    SELECT student,page FROM ".$self->table("namefield")." ) AS a"
+	." ON p.student=a.student AND p.page=a.page"
+	." WHERE p.student=?"},
        'studentPage'=>{'sql'=>"SELECT student,page FROM ".$self->table("page")
+		       ." WHERE markdiameter>0"
 		       ." LIMIT 1"},
        'dims'=>{'sql'=>"SELECT width,height,markdiameter FROM "
 		.$self->table("page")
@@ -377,6 +437,14 @@ sub define_statements {
 	." ON a.corner=b.corner"
 	."    AND (abs(a.x-b.x)>? OR abs(a.y-b.y)>?)"
 	." LIMIT 1"},
+       'AssocNumber'=>{'sql'=>"SELECT COUNT(*) FROM ".$self->table("association")},
+       'orientation'=>{'sql'=>"SELECT MIN(ratio) AS minratio,"
+		       ."             MAX(ratio) AS maxratio FROM "
+		       ." (SELECT CASE WHEN ABS(width)<1 THEN 1"
+		       ."              ELSE height/width END"
+		       ."         AS ratio"
+		       ."  FROM ".$self->table("page")
+		       ." )"},
       };
 }
 
@@ -466,6 +534,18 @@ sub page_info {
 	       $self->statement('pageInfo'),{},$student,$page));
 }
 
+# type_info($type,$student,$page) returns an array of HASH references
+# containing all fiels in the $type table ($type may equal digit, box
+# or namefield) corresponding to the $student,$page page.
+
+sub type_info {
+  my ($self,$type,$student,$page)=@_;
+  return(@{$self->dbh
+	     ->selectall_arrayref($self->statement($type.'Info'),
+				  {Slice=>{}},
+				  $student,$page)});
+}
+
 # pages_for_student($student,[%options]) returns a list of the page
 # numbers on the subject (starting from 1 for each student) for this
 # student. With 'select'=>'box' as an option, restricts to the pages
@@ -559,7 +639,7 @@ sub question_name {
 
 sub clear_all {
     my ($self)=@_;
-    for my $t (qw/page mark namefield box digit source/) {
+    for my $t (qw/page mark namefield box digit source association/) {
 	$self->sql_do("DELETE FROM ".$self->table($t));
     }
 }
@@ -623,4 +703,40 @@ sub get_box_flags {
 			   $student,$question,$answer));
 }
 
+# new_association($student,$id) adds an association to the
+# pre-association data (associations made before the exam)
+
+sub new_association {
+  my ($self,$student,$id)=@_;
+  $self->statement('NEWAssociation')->execute($student,$id);
+}
+
+# pre_association() returns the number of pre-asociations
+
+sub pre_association {
+  my ($self)=@_;
+  return($self->sql_single($self->statement("AssocNumber")));
+}
+
+# orientation() returns "portrait" or "landscape" if all pages have
+# the same orientation, and "" otherwise. In array context,
+# orientation() returns the min and max height/width ratio for all
+# pages.
+
+sub orientation {
+  my ($self)=@_;
+  my @ors=$self->sql_row($self->statement("orientation"));
+  if(wantarray) {
+    return(@ors);
+  } else {
+    if($ors[0] > 1.1) {
+      return("portrait");
+    } elsif($ors[1] < 0.9) {
+      return("landscape");
+    } else {
+      return("");
+    }
+  }
+}
+
 1;
