@@ -1,6 +1,6 @@
 # -*- perl -*-
 #
-# Copyright (C) 2011-2012 Alexis Bienvenue <paamc@passoire.fr>
+# Copyright (C) 2011-2013 Alexis Bienvenue <paamc@passoire.fr>
 #
 # This file is part of Auto-Multiple-Choice
 #
@@ -271,16 +271,21 @@ sub populate_from_xml {
 	if($question =~ /^[0-9]+$/) {
 	  my $q=$s->{'question'}->{$question};
 	  $self->question_title($question,$q->{'titre'});
-	  $self->statement('NEWQuestion')
-	    ->execute($student,$question,
-		      ($q->{'multiple'} ? QUESTION_MULT : QUESTION_SIMPLE),
-		      ($q->{'indicative'} ? 1 : 0),$q->{'bareme'});
+	  $self->new_question
+	    ($student,$question,
+	     ($q->{'multiple'} ? QUESTION_MULT : QUESTION_SIMPLE),
+	     ($q->{'indicative'} ? 1 : 0),$q->{'bareme'});
 
-	  for my $answer (keys %{$q->{'reponse'}}) {
-	    my $a=$q->{'reponse'}->{$answer};
-	    $self->statement('NEWAnswer')
-	      ->execute($student,$question,$answer,
-			$a->{'bonne'},$a->{'bareme'});
+	  if($q->{'reponse'}) {
+	    if(ref($q->{'reponse'}) eq 'HASH') {
+	      for my $answer (keys %{$q->{'reponse'}}) {
+		my $a=$q->{'reponse'}->{$answer};
+		$self->new_answer($student,$question,$answer,
+				  $a->{'bonne'},$a->{'bareme'});
+	      }
+	    } else {
+	      debug "WARNING: reponse is not a HASHREF for S=$student Q=$question";
+	    }
 	  }
 	} else {
 	  debug "Unknown question id: <$question>";
@@ -388,6 +393,21 @@ sub define_statements {
 		  ." (student,see) VALUES (?,?)"},
      'getAlias'=>{'sql'=>"SELECT see FROM ".$self->table("alias")
 		  ." WHERE student=?"},
+     'postCorrectNew'=>{'sql'=>"CREATE TEMPORARY TABLE IF NOT EXISTS"
+		       ." pc_temp (q INTEGER, a INTEGER, c REAL, PRIMARY KEY(q,a))"},
+     'postCorrectClr'=>{'sql'=>"DELETE FROM pc_temp"},
+     'postCorrectPop'=>{'sql'=>"INSERT INTO pc_temp (q,a,c) "
+			." SELECT id_a,id_b,CASE"
+			."   WHEN manual >= 0 THEN manual"
+			."   WHEN total<=0 THEN -1"
+			."   WHEN black >= ? * total THEN 1"
+			."   ELSE 0"
+			." END"
+			." FROM ".$self->table("zone","capture")
+			." WHERE capture_zone.student=? AND capture_zone.copy=? AND capture_zone.type=?"},
+     'postCorrectSet'=>{'sql'=>"UPDATE ".$self->table("answer")
+			." SET correct=(SELECT c FROM pc_temp"
+			."     WHERE q=question AND a=answer)"},
      'postCorrect'=>{'sql'=>"UPDATE ".$self->table("answer")
 		     ." SET correct="
 		     ."(SELECT CASE"
@@ -402,6 +422,8 @@ sub define_statements {
      'NEWScore'=>{'sql'=>"INSERT INTO ".$self->table("score")
 		  ." (student,copy,question,score,max,why)"
 		  ." VALUES (?,?,?,?,?,?)"},
+     'cancelScore'=>{'sql'=>"UPDATE ".$self->table("score")
+		     ." SET why=? WHERE student=? AND copy=? AND question=?"},
      'NEWMark'=>{'sql'=>"INSERT INTO ".$self->table("mark")
 		  ." (student,copy,total,max,mark)"
 		  ." VALUES (?,?,?,?,?)"},
@@ -453,6 +475,12 @@ sub define_statements {
      'codesCounts'=>{'sql'=>"SELECT student,copy,value,COUNT(*) as nb"
 		     ." FROM ".$self->table("code")
 		     ." WHERE code=? GROUP BY value"},
+     'preAssocCounts'=>
+     {'sql'=>"SELECT m.student,m.copy,l.id AS value,COUNT(*) AS nb"
+      ." FROM ".$self->table("mark")." AS m"
+      ."      , ".$self->table("association","layout")." AS l"
+      ." ON m.student=l.student AND m.copy=0"
+      ." GROUP BY l.id"},
 
      'avgMark'=>{'sql'=>"SELECT AVG(mark) FROM ".$self->table("mark")
 		 ." WHERE NOT (student=? AND copy=?)"},
@@ -485,6 +513,12 @@ sub define_statements {
       ." LEFT OUTER JOIN ".$self->table("title")." t"
       ." ON q.question=t.question"
       ." WHERE student=?"},
+     'deleteScores'=>{'sql'=>"DELETE FROM ".$self->table('score')
+		     ." WHERE student=? AND copy=?"},
+     'deleteMarks'=>{'sql'=>"DELETE FROM ".$self->table('mark')
+		    ." WHERE student=? AND copy=?"},
+     'deleteCodes'=>{'sql'=>"DELETE FROM ".$self->table('code')
+		    ." WHERE student=? AND copy=?"},
     };
 }
 
@@ -550,6 +584,15 @@ sub main_strategy_all {
   return(join(',',$self->sql_list($self->statement('getAllMain'),$student)));
 }
 
+# new_question($student,$question,$type,$indicative,$strategy) adds a
+# question in the database, giving its characteristics
+
+sub new_question {
+  my ($self,$student,$question,$type,$indicative,$strategy)=@_;
+  $self->statement('NEWQuestion')->execute
+    ($student,$question,$type,$indicative,$strategy);
+}
+
 # question_strategy($student,$question) returns the scoring strategy
 # string for a particlar question: argument of the \scoring command
 # used inside a question/questionmult environment, before the
@@ -560,6 +603,15 @@ sub question_strategy {
   return($self->sql_single($self->statement('qStrat'),$student,$question));
 }
 
+# new_answer($student,$question,$answer,$correct,$strategy) adds an
+# answer in the database, giving its characteristics
+
+sub new_answer {
+  my ($self,$student,$question,$answer,$correct,$strategy)=@_;
+  $self->statement('NEWAnswer')->execute
+    ($student,$question,$answer,$correct,$strategy);
+}
+
 # answer_strategy($student,$question,$answer) returns the scoring
 # strategy string for a particular answer: argument of the \scoring
 # command used after \correctchoice and \wrongchoice commands.
@@ -752,14 +804,36 @@ sub unalias {
 # only when the questions/answers are not different from a sheet to
 # another (contrary to the use of random numerical values for
 # exemple).
+#
+# Here are two SQL implementations for postcorrect: postcorrect_a acts
+# with a single SQL request, whereas postcorrect_b builds a temporary
+# table and requires several SQL requests. postcorrect_b is usualy the
+# fastest, so that postcorrect is only a relay to postcorrect_b. Note
+# that an UPDATE request on a JOINture should be a good idea, but this
+# is not allowed by SQLite.
 
-sub postcorrect {
+sub postcorrect_a {
   my ($self,$student,$copy,$darkness_threshold)=@_;
   $self->{'data'}->require_module('capture');
   $self->statement('postCorrect')
     ->execute($darkness_threshold,$student,$copy,ZONE_BOX);
 }
 
+sub postcorrect_b {
+  my ($self,$student,$copy,$darkness_threshold)=@_;
+  $self->{'data'}->require_module('capture');
+  $self->statement('postCorrectNew')->execute();
+  $self->statement('postCorrectClr')->execute();
+  $self->statement('postCorrectPop')
+    ->execute($darkness_threshold,$student,$copy,ZONE_BOX);
+  $self->statement('postCorrectSet')->execute();
+}
+
+sub postcorrect {
+  my ($self,@args)=@_;
+  $self->postcorrect_b(@args);
+}
+
 # new_score($student,$copy,$question,$score,$score_max,$why) adds a
 # question score row.
 
@@ -769,6 +843,15 @@ sub new_score {
     ->execute($student,$copy,$question,$score,$score_max,$why);
 }
 
+# cancel_score($student,$copy,$question) cancels scoring (sets the
+# score and maximum score to zero) for this question.
+
+sub cancel_score {
+  my ($self,$student,$copy,$question)=@_;
+  $self->statement('cancelScore')
+    ->execute('C',$student,$copy,$question);
+}
+
 # new_mark($student,$copy,$total,$max,$mark) adds a mark row.
 
 sub new_mark {
@@ -919,18 +1002,32 @@ sub student_scoring_base {
   my $r={'student_alias'=>$student_strategy,
 	 'questions'=>{},
 	 'main_strategy'=>$self->main_strategy_all($student_strategy)};
-  my $sth;
-  $sth=$self->statement('studentQuestionsBase');
-  $sth->execute($student_strategy);
-  while(my $qa=$sth->fetchrow_hashref) {
-    $r->{'questions'}->{$qa->{'question'}}=$qa;
-  }
-  $sth=$self->statement('studentAnswersBase');
-  $sth->execute($darkness_threshold,$student,$copy,ZONE_BOX,$student_strategy);
-  while(my $qa=$sth->fetchrow_hashref) {
-    push @{$r->{'questions'}->{$qa->{'question'}}->{'answers'}},$qa;
+  my @sid=($student);
+  push @sid,$student_strategy if($student != $student_strategy);
+  for my $s (@sid) {
+    my $sth;
+    $sth=$self->statement('studentQuestionsBase');
+    $sth->execute($s);
+    while(my $qa=$sth->fetchrow_hashref) {
+      $r->{'questions'}->{$qa->{'question'}}=$qa;
+    }
+    $sth=$self->statement('studentAnswersBase');
+    $sth->execute($darkness_threshold,$student,$copy,ZONE_BOX,$s);
+    while(my $qa=$sth->fetchrow_hashref) {
+      push @{$r->{'questions'}->{$qa->{'question'}}->{'answers'}},$qa;
+    }
   }
   return($r);
 }
 
+# delete_scoring_data($student,$copy) deletes all scoring data
+# relative to a particular answer sheet.
+
+sub delete_scoring_data {
+  my ($self,$student,$copy)=@_;
+  for my $part (qw/Scores Marks Codes/) {
+    $self->statement('delete'.$part)->execute($student,$copy);
+  }
+}
+
 1;
