@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2009-2012 Alexis Bienvenue <paamc@passoire.fr>
+# Copyright (C) 2009-2013 Alexis Bienvenue <paamc@passoire.fr>
 #
 # This file is part of Auto-Multiple-Choice
 #
@@ -22,6 +22,9 @@ package AMC::Export;
 use AMC::Basic;
 use AMC::Data;
 use AMC::NamesFile;
+use AMC::Messages;
+
+@ISA=("AMC::Messages");
 
 use_gettext;
 
@@ -42,6 +45,7 @@ sub new {
 	'noms.encodage'=>'',
 	'noms.separateur'=>'',
 	'noms.useall'=>1,
+	'noms.postcorrect'=>'',
 	'noms.abs'=>'ABS',
 	'noms.identifiant'=>'',
 
@@ -50,6 +54,8 @@ sub new {
 	'sort.keys'=>['s:student.name','n:student.line'],
 
 	'marks'=>[],
+
+	'messages'=>[],
     };
     bless ($self, $class);
     return $self;
@@ -94,6 +100,70 @@ sub load {
     }
 }
 
+sub question_group {
+  my ($self,$question)=@_;
+  if($question->{title} =~ /^([^:]+):/) {
+    return($1);
+  } else {
+    return(undef);
+  }
+}
+
+sub group_sum_q {
+  my ($self,%g)=@_;
+  return({question=>-1,%g,
+	  title=>"<".$g{group_sum}.">"});
+}
+
+sub insert_groups_sum_headers {
+  my ($self,@questions)=@_;
+  if($self->{'out.groupsums'}) {
+    my %group=();
+    my @r=();
+    for my $q (@questions) {
+      my $g=$self->question_group($q);
+      $q->{group}=$g if(defined($g));
+
+      if(defined($g) && $g eq $group{group_sum}) {
+	$group{indic0}=1 if($q->{indic0});
+	$group{indic1}=1 if($q->{indic1});
+	$group{n}++;
+      } else {
+	push @r,$self->group_sum_q(%group) if(defined($group{group_sum}));
+	%group=(group_sum=>$g,n=>1,
+		indic0=>$q->{indic0},indic1=>$q->{indic1});
+      }
+      push @r,$q;
+    }
+    push @r,$self->group_sum_q(%group) if(defined($group{group_sum}));
+    return(@r);
+  } else {
+    return(@questions);
+  }
+}
+
+sub test_indicative {
+  my ($self,$question)=@_;
+  for my $state (0,1) {
+    $question->{'indic'.$state}=1
+      if($self->{'_scoring'}->one_indicative($question->{question},$state));
+  }
+}
+
+sub codes_questions {
+  my ($self,$codes,$questions,$plain)=@_;
+  @$codes=$self->{'_scoring'}->codes();
+  if($plain) {
+    my $codes_re="(".join("|",map { "\Q$_\E" } @$codes).")";
+    @$questions=grep { $_->{'title'} !~ /^$codes_re\.[0-9]+$/ }
+      $self->{'_scoring'}->questions;
+  } else {
+    @$questions=$self->{'_scoring'}->questions;
+  }
+  for(@$questions) { $self->test_indicative($_); }
+  @$questions=$self->insert_groups_sum_headers(@$questions);
+}
+
 sub pre_process {
     my ($self)=@_;
 
@@ -115,7 +185,8 @@ sub pre_process {
     my $sth=$self->{'_scoring'}->statement('marks');
     $sth->execute;
   STUDENT: while(my $m=$sth->fetchrow_hashref) {
-      next STUDENT if($m->{student}==$post_correct[0] &&
+      next STUDENT if((!$self->{'noms.postcorrect'}) &&
+		      $m->{student}==$post_correct[0] &&
 		      $m->{'copy'}==$post_correct[1]);
 
       $m->{'abs'}=0;
