@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2009-2012 Alexis Bienvenue <paamc@passoire.fr>
+# Copyright (C) 2009-2013 Alexis Bienvenue <paamc@passoire.fr>
 #
 # This file is part of Auto-Multiple-Choice
 #
@@ -68,6 +68,8 @@ sub export {
 
     $sep="\t" if($sep =~ /^tab$/i);
 
+    $self->{'noms.postcorrect'}=($self->{'out.ticked'} ne '');
+
     $self->pre_process();
 
     open(OUT,">:encoding(".$self->{'out.encodage'}.")",$fichier);
@@ -95,8 +97,9 @@ sub export {
 
     push @columns,map { translate_column_title($_); } ("note");
 
-    my @questions=$self->{'_scoring'}->questions;
-    my @codes=$self->{'_scoring'}->codes;
+    my @codes;
+    my @questions;
+    $self->codes_questions(\@codes,\@questions,!$self->{'out.ticked'});
 
     if($self->{'out.ticked'}) {
       push @columns,map { ($_->{'title'},"TICKED:".$_->{'title'}) } @questions;
@@ -132,7 +135,7 @@ sub export {
 	    my $t='';
 	    my @tl=$self->{'_capture'}
 	      ->ticked_list(@sc,$q->{'question'},$dt);
-	    if($self->{'_scoring'}->multiple($m->{'student'},$q->{'question'})) {
+	    if($self->{_capture}->has_answer_zero(@sc,$q->{'question'})) {
 	      if(shift @tl) {
 		$t.='0';
 	      }
