@@ -39,6 +39,7 @@ sub new {
     $self->{'out.size'}="10";
     $self->{'out.stats'}='';
     $self->{'out.statsindic'}='';
+    $self->{'out.groupsums'}='';
     if(can_load(modules=>{'Gtk2'=>undef,'Cairo'=>undef})) {
       debug "Using Gtk2/Cairo to compute column width";
       $self->{'calc.Gtk2'}=1;
@@ -187,6 +188,7 @@ my %style_col=(qw/student.key CodeA
 	       NOTE NoteF
 	       student.copy NumCopie
 	       TOTAL NoteQ
+	       GS NoteGS
 	       MAX NoteQ
 	       HEAD General
 	       /);
@@ -330,6 +332,12 @@ sub build_stats_table {
 
 }
 
+sub ods_locked_file {
+  my ($self,$fichier)=@_;
+  $fichier =~ s:([^/]+)$:.~lock.$1\#:;
+  return(-e $fichier);
+}
+
 sub export {
     my ($self,$fichier)=@_;
 
@@ -465,6 +473,15 @@ sub export {
 			 },
 			 'references'=>{'style:data-style-name' => 'Percentage'},
 			 );
+    # QpcGS : pourcentage de reussite global pour un groupe
+    $styles->createStyle('QpcGS',
+			 parent=>'Qpc',
+			 family=>'table-cell',
+			 properties=>{
+			     -area => 'table-cell',
+			     'fo:background-color'=>"#c4eeba",
+			 },
+			 );
 
     # StatsQName : nom de question
     $styles->createStyle('StatsQName',
@@ -593,6 +610,17 @@ sub export {
 			 'references'=>{'style:data-style-name' => 'NombreVide'},
 			 );
 
+    # NoteC : question annulee (par un allowempty)
+    $styles->createStyle('NoteC',
+			 parent=>'NoteQ',
+			 family=>'table-cell',
+			 properties=>{
+			     -area => 'table-cell',
+			     'fo:background-color'=>"#b1e3e9",
+			 },
+			 'references'=>{'style:data-style-name' => 'NombreVide'},
+			 );
+
     # NoteE : note car erreur "de syntaxe"
     $styles->createStyle('NoteE',
 			 parent=>'NoteQ',
@@ -604,6 +632,17 @@ sub export {
 			 'references'=>{'style:data-style-name' => 'NombreVide'},
 			 );
 
+    # NoteGS : score total pour un groupe
+    $styles->createStyle('NoteGS',
+			 parent=>'NoteQ',
+			 family=>'table-cell',
+			 properties=>{
+			     -area => 'table-cell',
+			     'fo:background-color'=>"#c4eeba",
+			 },
+			 'references'=>{'style:data-style-name' => 'DeuxDecimales'},
+			 );
+
     # NoteX : pas de note car la question ne figure pas dans cette copie la
     $styles->createStyle('NoteX',
 			 parent=>'Tableau',
@@ -739,16 +778,24 @@ sub export {
 			 },
 			 );
 
+    # EnteteGS : en-tete pour les groupes
+    $styles->createStyle('EnteteGS',
+			 parent=>'EnteteVertical',
+			 family=>'table-cell',
+			 properties=>{
+			     -area => 'table-cell',
+			     'fo:background-color'=>"#c4eeba",
+			 },
+			 );
+
     my @student_columns=split(/,+/,$self->{'out.columns'});
 
-    my @codes=$self->{'_scoring'}->codes;
-    my $codes_re="(".join("|",map { "\Q$_\E" } @codes).")";
-    my @questions=grep { $_->{'title'} !~ /$codes_re\.[0-9]+$/ }
-      $self->{'_scoring'}->questions;
-    my @questions_0=grep { $self->{'_scoring'}->one_indicative($_->{'question'},0); }
-      @questions;
-    my @questions_1=grep { $self->{'_scoring'}->one_indicative($_->{'question'},1); }
-      @questions;
+    my @codes;
+    my @questions;
+    $self->codes_questions(\@codes,\@questions,1);
+
+    my @questions_0=grep { $_->{indic0} } @questions;
+    my @questions_1=grep { $_->{indic1} } @questions;
 
     debug "Questions: ".join(', ',map { $_->{'question'}.'='.$_->{'title'} } @questions);
     debug "Questions PLAIN: ".join(', ',map { $_->{'title'} } @questions_0);
@@ -812,7 +859,8 @@ sub export {
 
     for(@questions_0) {
 	$doc->columnStyle($feuille,$ii,'col.notes');
-	$doc->cellStyle($feuille,$y0,$ii,'EnteteVertical');
+	$doc->cellStyle($feuille,$y0,$ii,
+			($_->{group_sum} ? 'EnteteGS' : 'EnteteVertical'));
 	$doc->cellValue($feuille,$y0,$ii++,encode('utf-8',$_->{'title'}));
     }
     for(@questions_1) {
@@ -844,10 +892,14 @@ sub export {
 
     $ii=$x1;
     for(@questions_0) {
-      $doc->cellStyle($feuille,$jj,$ii,'NoteQ');
-      $doc->cellValueType($feuille,$jj,$ii,'float');
-      $doc->cellValue($feuille,$jj,$ii++,
-		      $self->{'_scoring'}->question_maxmax($_->{'question'}));
+      if(defined($_->{group_sum})) {
+	$ii++;
+      } else {
+	$doc->cellStyle($feuille,$jj,$ii,'NoteQ');
+	$doc->cellValueType($feuille,$jj,$ii,'float');
+	$doc->cellValue($feuille,$jj,$ii++,
+			$self->{'_scoring'}->question_maxmax($_->{'question'}));
+      }
     }
 
     $code_row{'max'}=$jj;
@@ -871,6 +923,7 @@ sub export {
     my @presents=();
     my %scores;
     my @scores_columns;
+    my %group_single=();
 
     my $y1=$jj+1;
 
@@ -935,36 +988,65 @@ sub export {
 
 	# second: columns for all questions scores
 
+	my @group_columns=();
+	my $group_maxsum=0;
+
 	for my $q (@questions_0,@questions_1) {
 	  if($m->{'abs'}) {
 	    $doc->cellStyle($feuille,$jj,$ii,'NoteX');
 	  } else {
-	    my $r=$self->{'_scoring'}
-	      ->question_result($m->{'student'},$m->{'copy'},$q->{'question'});
-	    $doc->cellValueType($feuille,$jj,$ii,'float');
-	    if($self->{'_scoring'}->indicative($m->{'student'},$q->{'question'})) {
-	      $doc->cellStyle($feuille,$jj,$ii,'CodeV');
+	    if(defined($q->{group_sum})) {
+	      if(@group_columns) {
+		if(defined($group_single{$q->{group_sum}})) {
+		  $group_single{$q->{group_sum}}->{ok}=0
+		    if($group_maxsum != $group_single{$q->{group_sum}}->{maxsum});
+		} else {
+		  $group_single{$q->{group_sum}}=
+		    {ii=>$ii,maxsum=>$group_maxsum,ok=>1};
+		}
+		set_cell($doc,$feuille,$jj,$ii,$m->{'abs'},
+			 'GS','','numeric'=>1,
+			 'formula'=>"oooc:=SUM(".subrow_condensed($jj,@group_columns).")");
+		push @{$col_cells{$ii}},$jj;
+	      } else {
+		$doc->cellStyle($feuille,$jj,$ii,'NoteX');
+	      }
+	      @group_columns=();
+	      $group_maxsum=0;
 	    } else {
-	      if(defined($r->{'score'})) {
-		if(!$scores{$q->{'question'}}) {
-		  $scores{$q->{'question'}}=1;
-		  push @scores_columns,$ii;
-		  push @{$col_cells{$ii}},$jj;
-		  if($r->{'why'} =~ /v/i) {
-		    $doc->cellStyle($feuille,$jj,$ii,'NoteV');
-		  } elsif($r->{'why'} =~ /e/i) {
-		    $doc->cellStyle($feuille,$jj,$ii,'NoteE');
+	      my $r=$self->{'_scoring'}
+		->question_result($m->{'student'},$m->{'copy'},$q->{'question'});
+	      $doc->cellValueType($feuille,$jj,$ii,'float');
+	      if($self->{'_scoring'}->indicative($m->{'student'},$q->{'question'})) {
+		$doc->cellStyle($feuille,$jj,$ii,'CodeV');
+	      } else {
+		if(defined($r->{'score'})) {
+		  if(!$scores{$q->{'question'}}) {
+		    $scores{$q->{'question'}}=1;
+		    push @scores_columns,$ii;
+		    push @{$col_cells{$ii}},$jj;
+		    if($q->{group}) {
+		      push @group_columns,$ii;
+		      $group_maxsum+=$r->{max};
+		    }
+		    if($r->{'why'} =~ /c/i) {
+		      $doc->cellStyle($feuille,$jj,$ii,'NoteC');
+		    } elsif($r->{'why'} =~ /v/i) {
+		      $doc->cellStyle($feuille,$jj,$ii,'NoteV');
+		    } elsif($r->{'why'} =~ /e/i) {
+		      $doc->cellStyle($feuille,$jj,$ii,'NoteE');
+		    } else {
+		      $doc->cellStyle($feuille,$jj,$ii,'NoteQ');
+		    }
 		  } else {
-		    $doc->cellStyle($feuille,$jj,$ii,'NoteQ');
+		    $doc->cellStyle($feuille,$jj,$ii,'NoteX');
 		  }
 		} else {
 		  $doc->cellStyle($feuille,$jj,$ii,'NoteX');
 		}
-	      } else {
-		$doc->cellStyle($feuille,$jj,$ii,'NoteX');
 	      }
+	      $doc->cellValue($feuille,$jj,$ii,$r->{'score'});
 	    }
-	    $doc->cellValue($feuille,$jj,$ii,$r->{'score'});
 	  }
 	  $ii++;
 	}
@@ -1006,6 +1088,24 @@ sub export {
 
     $self->{'_scoring'}->end_transaction('XODS');
 
+    ##########################################################################
+    # back to row for groups max
+    ##########################################################################
+
+    for my $g (keys %group_single) {
+      my $j0=$code_row{'max'};
+      my $i0=$group_single{$g}->{ii};
+      if($group_single{$g}->{ok}) {
+	$doc->cellStyle($feuille,$j0,$i0,'NoteGS');
+	$doc->cellValueType($feuille,$j0,$i0,'float');
+	$doc->cellValue($feuille,$j0,$i0,$group_single{$g}->{maxsum});
+	$doc->cellStyle($feuille,$code_row{'average'},$i0,'QpcGS');
+      } else {
+	$doc->cellStyle($feuille,$j0,$i0,'NoteX');
+	$doc->cellStyle($feuille,$code_row{'average'},$i0,'NoteX');
+      }
+    }
+
     ##########################################################################
     # try to set right column width
     ##########################################################################
@@ -1062,7 +1162,7 @@ sub export {
     ##########################################################################
 
 # TRANSLATORS: Label of the table with a legend (explaination of the colors used) in the exported ODS spreadsheet.
-    my $legend=$doc->appendTable(encode('utf-8',__("Legend")),8,2);
+    my $legend=$doc->appendTable(encode('utf-8',__("Legend")),9,2);
 
     $doc->cellSpan($legend,0,0,2);
     $doc->cellStyle($legend,0,0,'Titre');
@@ -1078,6 +1178,10 @@ sub export {
 # TRANSLATORS: From the legend in the exported ODS spreadsheet. This refers to the questions that have not been answered.
     $doc->cellValue($legend,$jj,1,encode('utf-8',__("No answer")));
     $jj++;
+    $doc->cellStyle($legend,$jj,0,'NoteC');
+# TRANSLATORS: From the legend in the exported ODS spreadsheet. This refers to the questions that have not been answered, but are cancelled by the use of allowempty scoring strategy.
+    $doc->cellValue($legend,$jj,1,encode('utf-8',__("Cancelled")));
+    $jj++;
     $doc->cellStyle($legend,$jj,0,'NoteE');
 # TRANSLATORS: From the legend in the exported ODS spreadsheet. This refers to the questions that got an invalid answer.
     $doc->cellValue($legend,$jj,1,encode('utf-8',__("Invalid answer")));
@@ -1121,6 +1225,10 @@ sub export {
 
     $archive->save;
 
+    if($self->ods_locked_file($fichier)) {
+      $self->add_message('INFO',__("An old state of the exported file seems to be already opened. Use File/Reload from OpenOffice/LibreOffice to refresh."));
+    }
+
 }
 
 1;
