@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2008-2011 Alexis Bienvenue <paamc@passoire.fr>
+# Copyright (C) 2008-2012 Alexis Bienvenue <paamc@passoire.fr>
 #
 # This file is part of Auto-Multiple-Choice
 #
@@ -38,20 +38,22 @@ sub new {
     }
 
     if(! $self->{'traitement'}) {
-	if($self->{'mode'} eq 'auto') {
-	    if(-f amc_specdir('lib').'/AMC-detect') {
-		$self->{'mode'}='opencv';
-	    } else {
-		$self->{'mode'}='manual';
-	    }
-	}
-	if($self->{'mode'} eq 'opencv') {
-	    $self->{'traitement'}=amc_specdir('lib').'/AMC-detect';
-	} elsif($self->{'mode'} eq 'manual') {
-	    $self->{'traitement'}=amc_specdir('lib').'/AMC-traitement-image';
+      if($self->{'mode'} eq 'auto') {
+	if(-f amc_specdir('libexec').'/AMC-detect') {
+	  $self->{'mode'}='opencv';
 	} else {
-	    die "AMC::Image: No program to execute";
+	  $self->{'mode'}='manual';
 	}
+      }
+      if($self->{'mode'} eq 'opencv') {
+	$self->{'traitement'}=amc_specdir('libexec').'/AMC-detect';
+      } elsif($self->{'mode'} eq 'manual') {
+	$self->{'traitement'}=amc_specdir('libexec').'/AMC-traitement-image';
+      }
+    }
+
+    if(! -f $self->{'traitement'}) {
+      die "AMC::Image: No program to execute";
     }
 
     bless $self;
@@ -90,11 +92,12 @@ sub commande {
 
     print { $self->{'ipc_in'} } join(' ',@cmd)."\n";
 
-  GETREPONSE: while($_=readline($self->{'ipc_out'})) {
-      chomp;
-      debug "|> $_";
-      last GETREPONSE if(/_{2}END_{2}/);
-      push @r,$_;
+    my $o;
+  GETREPONSE: while($o=readline($self->{'ipc_out'})) {
+      chomp($o);
+      debug "|> $o";
+      last GETREPONSE if($o =~ /_{2}END_{2}/);
+      push @r,$o;
   }
 
     return(@r);
