@@ -68,9 +68,10 @@ sub load {
 
     $self->{'noms'}=[];
 
-    debug "Reading names file $self->{'fichier'}";
+    if($self->{'fichier'} &&
+       -f $self->{'fichier'} && ! -z $self->{'fichier'}) {
 
-    if(-f $self->{'fichier'} && ! -z $self->{'fichier'}) {
+      debug "Reading names file $self->{'fichier'}";
 
       # First pass: detect the number of comment lines, and the
       # separator
@@ -106,6 +107,10 @@ sub load {
 
       # Second pass: read the file with Text::CSV
 
+      $self->{'numeric.content'}={};
+      $self->{'simple.content'}={};
+      $self->{duplicates}={};
+
       my $io;
       if(open($io,"<:encoding(".$self->{'encodage'}.")",
 	      $self->{'fichier'})) {
@@ -124,7 +129,12 @@ sub load {
 	# first line: header
 
 	$self->{'heads'}=$csv->getline ($io);
-	if(!$self->{'heads'}) {
+	if($self->{'heads'}) {
+	  for my $h (@{$self->{'heads'}}) {
+	    $self->{'numeric.content'}->{$h}=0;
+	    $self->{'simple.content'}->{$h}=0;
+	  }
+	} else {
 	  debug("CSV [SEP=$sep]: Can't read headers");
 	  return(1,$.);
 	}
@@ -132,9 +142,6 @@ sub load {
 
 	# following lines
 
-	$self->{'numeric.content'}={};
-	$self->{'simple.content'}={};
-
 	my $csv_line=0;
 
 	while (my $row = $csv->getline_hr ($io)) {
@@ -150,6 +157,12 @@ sub load {
 	    if($ok) {
 	      $csv_line++;
 	      for my $k (keys %$row) {
+		if(defined($data{$k}->{$row->{$k}})
+		   && $data{$k}->{$row->{$k}}==1
+		   && !$self->{duplicates}->{$k} ) {
+		  $self->{duplicates}->{$k}=
+		    {content=>$row->{$k},line=>$csv_line};
+		}
 		$data{$k}->{$row->{$k}}++;
 		$self->{'numeric.content'}->{$k} ++
 		  if($row->{$k} =~ /^[ 0-9.+-]*$/i);
@@ -175,8 +188,12 @@ sub load {
 
 	# find unique identifiers
 
-	$self->{'keys'}=[grep { my @lk=(keys %{$data{$_}});
-				$#lk==$#{$self->{'noms'}}; }
+	for my $h (@{$self->{'heads'}}) {
+	  my @lk=(keys %{$data{$h}});
+	  $self->{duplicates}->{$h}->{n}=$#{$self->{'noms'}}-$#lk;
+	}
+
+	$self->{'keys'}=[grep { $self->{duplicates}->{$_}->{n} == 0 }
 			 @{$self->{'heads'}}];
 
 	# rajout identifiant
@@ -189,6 +206,8 @@ sub load {
       }
     } else {
 	debug("Inexistant or empty names list file");
+	debug("(file=".$self->{'fichier'}.")")
+	  if(defined($self->{'fichier'}));
 	$self->{'heads'}=[];
 	$self->{'keys'}=[];
 	$self->{'problems'}={'ID.dup'=>[],'ID.empty'=>0};
@@ -196,6 +215,16 @@ sub load {
     }
 }
 
+sub head_first_duplicate {
+  my ($self,$key)=@_;
+  return($self->{'duplicates'}->{$key}->{content});
+}
+
+sub head_n_duplicates {
+  my ($self,$key)=@_;
+  return($self->{duplicates}->{$key}->{n});
+}
+
 sub get_value {
     my ($self,$key,$vals)=@_;
     my $r='';
@@ -281,6 +310,18 @@ sub keys { # entetes qui peuvent servir de cle unique
 		  @{$self->{'keys'}});
 }
 
+sub heads_for_keys { # entetes qui peuvent servir de cle unique
+    my ($self)=@_;
+    return(sort { $self->{duplicates}->{$a}->{n} <=>
+		    $self->{duplicates}->{$b}->{n}
+		  || $self->{'simple.content'}->{$b} <=>
+		      $self->{'simple.content'}->{$a}
+		  || $self->{'numeric.content'}->{$b} <=>
+		      $self->{'numeric.content'}->{$a}
+		  || $a cmp $b }
+		  @{$self->{'heads'}});
+}
+
 sub liste {
     my ($self,$head)=@_;
     return(map { $_->{$head} } @{$self->{'noms'}} );
