@@ -1,6 +1,6 @@
 #! /usr/bin/perl -w
 #
-# Copyright (C) 2012 Alexis Bienvenue <paamc@passoire.fr>
+# Copyright (C) 2012-2013 Alexis Bienvenue <paamc@passoire.fr>
 #
 # This file is part of Auto-Multiple-Choice
 #
@@ -22,6 +22,9 @@ package AMC::Test;
 
 use AMC::Basic;
 use AMC::Data;
+use AMC::DataModule::capture qw(:zone);
+use AMC::DataModule::scoring qw(:question);
+use AMC::Scoring;
 
 use Text::CSV;
 use File::Spec::Functions qw(tmpdir);
@@ -55,6 +58,7 @@ sub new {
      'perfect_copy'=>[3],
      'src'=>'',
      'debug'=>0,
+     'debug_pixels'=>0,
      'scans'=>'',
      'seuil'=>0.5,
      'bw_threshold'=>0.6,
@@ -71,6 +75,7 @@ sub new {
      'annote'=>'',
      'annote_files'=>[],
      'annote_ascii'=>0,
+     'annote_position'=>'marge',
      'verdict'=>'%(id) %(ID)'."\n".'TOTAL : %S/%M => %s/%m',
      'verdict_question'=>"\"%"."s/%"."m\"",
      'model'=>'(N).pdf',
@@ -78,7 +83,11 @@ sub new {
      'ok_checksums_file'=>'',
      'to_check'=>[],
      'export_full_csv'=>[],
+     'export_csv_ticked'=>'AB',
      'blind'=>0,
+     'check_zooms'=>{},
+     'skip_prepare'=>0,
+     'skip_scans'=>0,
     };
 
   for (keys %oo) {
@@ -159,11 +168,24 @@ sub install {
     }
   }
 
-  for my $d (qw(data cr cr/zooms cr/corrections cr/corrections/jpg cr/corrections/pdf)) {
+  for my $d (qw(data cr cr/corrections cr/corrections/jpg cr/corrections/pdf scans)) {
     mkdir($self->{'temp_dir'}."/$d") if(!-d $self->{'temp_dir'}."/$d");
   }
 
   $self->{'debug_file'}=$self->{'temp_dir'}."/debug.log";
+  open(DB,">",$self->{'debug_file'});
+  print DB "Test\n";
+  close(DB);
+}
+
+sub see_blob {
+  my ($self,$name,$blob)=@_;
+  my $path=$self->{'temp_dir'}.'/'.$name;
+  open FILE,">$path";
+  binmode(FILE);
+  print FILE $blob;
+  close FILE;
+  $self->see_file($path);
 }
 
 sub see_file {
@@ -239,11 +261,11 @@ sub prepare {
 		     '--n-copies',$self->{'n_copies'},
 		     '--prefix',$self->{'temp_dir'}.'/',
 		     '%PROJ/'.$self->{'src'},
-		     );
+		    );
   $self->amc_command('meptex',
 		     '--src','%PROJ/calage.xy',
 		     '--data','%DATA',
-		     );
+		    );
   $self->amc_command('prepare',
 		     '--filter',$self->{'filter'},
 		     '--with',$self->{'tex_engine'},
@@ -251,7 +273,7 @@ sub prepare {
 		     '--n-copies',$self->{'n_copies'},
 		     '--data','%DATA',
 		     '%PROJ/'.$self->{'src'},
-		     );
+		    );
 }
 
 sub analyse {
@@ -278,15 +300,29 @@ sub analyse {
 		       '--fich-numeros',$nf,
 		       '--data','%DATA',
 		      );
-    system("cd $self->{'temp_dir'} ; gm convert xx-*.pdf yy-scan.png");
 
     opendir(my $dh, $self->{'temp_dir'})
       || die "can't opendir $self->{'temp_dir'}: $!";
-    my @s = grep { /^yy-scan\./ } readdir($dh);
+    my @s = grep { /^xx-copie-/ } readdir($dh);
     closedir $dh;
     push @{$self->{'scans'}},map { $self->{'temp_dir'}."/$_" } @s;
   }
 
+  # prepares a file with the scans list
+
+  my $scans_list=$self->{'temp_dir'}."/scans-list.txt";
+  open(SL,">",$scans_list) or die "Open $scans_list: $!";
+  for(@{$self->{'scans'}}) { print SL "$_\n"; }
+  close(SL);
+
+  #
+
+  $self->amc_command('getimages',
+		     '--list',$scans_list,
+		     '--copy-to',$self->{'temp_dir'}."/scans",
+		     '--orientation',$self->get_orientation(),
+		     );
+
   $self->amc_command('analyse',
 		     ($self->{'multiple'} ? '--multiple' : '--no-multiple'),
 		     '--bw-threshold',$self->{'bw_threshold'},
@@ -295,16 +331,18 @@ sub analyse {
 		     '--projet','%PROJ',
 		     '--data','%DATA',
 		     '--debug-image-dir','%PROJ/cr',
-		     @{$self->{'scans'}},
+		     '--liste-fichiers',$scans_list,
 		     ) if($self->{'debug'});
   $self->amc_command('analyse',
 		     ($self->{'multiple'} ? '--multiple' : '--no-multiple'),
 		     '--bw-threshold',$self->{'bw_threshold'},
 		     '--pre-allocate',$self->{'pre_allocate'},
 		     '--tol-marque',$self->{'tol_marque'},
+		     ($self->{'debug'} || $self->{debug_pixels}
+		      ? '--debug-pixels' : '--no-debug-pixels'),
 		     '--projet','%PROJ',
 		     '--data','%DATA',
-		     @{$self->{'scans'}},
+		     '--liste-fichiers',$scans_list,
 		     );
 }
 
@@ -327,10 +365,17 @@ sub assoc {
 
   return if(!$self->{'list'});
 
+  my @code=();
+  if($self->{'code'} eq '<preassoc>') {
+    push @code,'--pre-association';
+  } else {
+    push @code,'--notes-id',$self->{'code'};
+  }
+
   $self->amc_command('association-auto',
 		     '--liste','%PROJ/'.$self->{'list'},
 		     '--liste-key',$self->{'list_key'},
-		     '--notes-id',$self->{'code'},
+		     @code,
 		     '--data','%DATA',
 		     );
 }
@@ -349,6 +394,16 @@ sub get_marks {
   }
 }
 
+sub get_orientation {
+  my ($self)=@_;
+
+  my $l=AMC::Data->new($self->{'temp_dir'}."/data")->module('layout');
+  $l->begin_read_transaction('tgor');
+  my $o=$l->orientation();
+  $l->end_transaction('tgor');
+  return($o);
+}
+
 sub check_perfect {
   my ($self)=@_;
   return if(!$self->{'perfect_copy'});
@@ -425,6 +480,12 @@ sub find_assoc {
   return($r);
 }
 
+sub compare {
+  my ($a,$b)=@_;
+  return( (($a eq 'x') && (!defined($b)))
+	  || ( $a eq $b ));
+}
+
 sub check_assoc {
   my ($self)=@_;
   return if(!$self->{'check_assoc'});
@@ -437,7 +498,9 @@ sub check_assoc {
   for my $m (@{$self->{'association'}}) {
     my $st=studentids_string($m->{'student'},$m->{'copy'});
     delete($p{$st})
-      if($self->{'check_assoc'}->{$st} eq $m->{'auto'});
+      if(compare($self->{'check_assoc'}->{$st},$m->{'auto'}));
+    delete($p{'m:'.$st})
+      if(compare($self->{'check_assoc'}->{'m:'.$st},$m->{'manual'}));
   }
 
   my @no=(keys %p);
@@ -456,6 +519,7 @@ sub annote {
 		     '--fich-noms','%PROJ/'.$self->{'list'},
 		     '--verdict',$self->{'verdict'},
 		     '--verdict-question',$self->{'verdict_question'},
+		     '--position',$self->{'annote_position'},
 		     '--projet','%PROJ',
 		     '--data','%DATA',
 		     );
@@ -535,7 +599,7 @@ sub check_export {
 		       '--module','CSV',
 		       '--fich-noms','%PROJ/'.$self->{'list'},
 		       '--option-out','columns=student.copy',
-		       '--option-out','ticked=AB',
+		       '--option-out','ticked='.$self->{export_csv_ticked},
 		       '-o','%PROJ/export.csv',
 		      );
     my $c=Text::CSV->new();
@@ -579,6 +643,81 @@ sub check_export {
   }
 }
 
+sub check_zooms {
+  my ($self)=@_;
+  my $cz=$self->{'check_zooms'};
+  my @zk=keys %$cz;
+  return if(!@zk);
+
+  my $capture=AMC::Data->new($self->{'temp_dir'}."/data")->module('capture');
+  $capture->begin_read_transaction('cZOO');
+
+  for my $p (keys %{$cz}) {
+    $self->trace("[T] Zooms check : $p");
+
+    my ($student,$page,$copy);
+    if($p =~ /^([0-9]+)-([0-9]+):([0-9]+)$/) {
+      $student=$1;$page=$2;$copy=$3;
+    } elsif($p =~ /^([0-9]+)-([0-9]+)$/) {
+      $student=$1;$page=$2;$copy=0;
+    }
+
+    my @zooms=grep { $_->{imagedata} }
+      (@{$capture->dbh
+	   ->selectall_arrayref($capture->statement('pageZonesDI'),
+				{Slice=>{}},
+				$student,$page,$copy,ZONE_BOX)
+	 });
+
+    if(1+$#zooms == $cz->{$p}) {
+      for(@zooms) {
+	$self->see_blob("zoom-".$student."-".$page.":".$copy."--"
+			.$_->{id_a}."-".$_->{id_b}.".png",$_->{imagedata});
+      }
+    } else {
+      $self->trace("[E] Zooms dir $p contains ".(1+$#zooms)." elements, but needs ".$cz->{$p});
+      exit(1);
+    }
+  }
+
+  $capture->end_transaction('cZOO');
+
+}
+
+sub check_textest {
+  my ($self,$tex_file)=@_;
+  if(!$tex_file) {
+    opendir(my $dh, $self->{'dir'})
+      || die "can't opendir $self->{'dir'}: $!";
+    my @tex = grep { /\.tex$/ } readdir($dh);
+    closedir $dh;
+    $tex_file=$tex[0] if(@tex);
+  }
+  $tex_file=$self->{'temp_dir'}."/".$tex_file;
+  if(-f $tex_file) {
+    open(TEX,"-|",$self->{'tex_engine'},
+	 $tex_file);
+    while(<TEX>) {
+      if(/^\!/) {
+	$self->trace("[E] latex error: $_");
+	exit(1);
+      }
+      if(/^SECTION\((.*)\)/) {
+	$self->end();
+	$self->begin($1);
+      }
+      if(/^TEST\(([^,]*),([^,]*)\)/) {
+	$self->test($1,$2);
+      }
+    }
+    close(TEX);
+    $self->end();
+  } else {
+    $self->trace("[X] TeX file not found: $tex_file");
+    exit(1);
+  }
+}
+
 sub data {
   my ($self)=@_;
   return(AMC::Data->new($self->{'temp_dir'}."/data"));
@@ -593,7 +732,9 @@ sub begin {
 
 sub end {
   my ($self)=@_;
-  $self->trace("[T] ".$self->{'test_title'}) if($self->{'test_title'});
+  $self->trace("[T] ".
+	       ($self->{'n.subt'} ? "(".$self->{'n.subt'}.") " : "")
+	       .$self->{'test_title'}) if($self->{'test_title'});
   $self->{'test_title'}='';
 }
 
@@ -611,7 +752,7 @@ sub datadump {
 sub test {
   my ($self,$x,$v,$subtest)=@_;
   if(!defined($subtest)) {
-    $subtest=$self->{'n.subt'}++;
+    $subtest=++$self->{'n.subt'};
   }
   if(ref($x) eq 'ARRAY') {
     for my $i (0..$#$x) {
@@ -636,12 +777,70 @@ sub test_undef {
   }
 }
 
+sub test_scoring {
+  my ($self,$question,$answers,$target_score)=@_;
+
+  my $data=AMC::Data->new($self->{'temp_dir'}."/data");
+  my $s=$data->module('scoring');
+  my $c=$data->module('capture');
+
+  $s->begin_transaction('tSCO');
+
+  $s->clear_strategy;
+  $s->clear_score;
+  $s->new_question(1,1,
+		   ($question->{multiple} ?
+		    QUESTION_MULT : QUESTION_SIMPLE),
+		   0,$question->{strategy});
+  my $i=0;
+  my $none=1;
+  my $none_t=1;
+  for my $a (@$answers) {
+    $i++ if(!$a->{noneof});
+    $none=0 if($a->{correct});
+    $s->new_answer(1,1,$i,$a->{correct},$a->{strategy});
+    $none_t=0 if($a->{ticked});
+    $c->set_zone_manual(1,1,0,
+			ZONE_BOX,1,$i,$a->{ticked});
+  }
+  if($question->{noneof_auto}) {
+    $s->new_answer(1,1,0,$none,'');
+    $c->set_zone_manual(1,1,0,
+			ZONE_BOX,1,0,$none_t);
+  }
+
+  my $qdata=$s->student_scoring_base(1,0,0.5);
+
+  $s->end_transaction('tSCO');
+
+  my $scoring=AMC::Scoring->new(data=>$self->{'temp_dir'}."/data");
+
+  set_debug($self->{debug_file});
+
+  $scoring->prepare_question($qdata->{questions}->{1});
+  my ($score,$why)=$scoring->score_question(1,0,$qdata->{questions}->{1},0);
+
+  set_debug('');
+
+  $self->test($score,$target_score);
+}
+
+sub update_sqlite {
+  my ($self)=@_;
+  my $d=AMC::Data->new($self->{'temp_dir'}."/data");
+  for my $m (qw/layout capture scoring association report/) {
+    $d->module($m);
+  }
+  return($self);
+}
+
 sub default_process {
   my ($self)=@_;
 
-  $self->prepare;
+  $self->prepare if(!$self->{skip_prepare});
   $self->defects;
-  $self->analyse;
+  $self->analyse if(!$self->{skip_scans});
+  $self->check_zooms;
   $self->note;
   $self->assoc;
   $self->get_assoc;
