@@ -1,6 +1,6 @@
 #! /usr/bin/perl
 #
-# Copyright (C) 2012 Alexis Bienvenue <paamc@passoire.fr>
+# Copyright (C) 2012-2013 Alexis Bienvenue <paamc@passoire.fr>
 #
 # This file is part of Auto-Multiple-Choice
 #
@@ -21,5 +21,9 @@
 use AMC::Test;
 
 AMC::Test->new('dir'=>__FILE__,'filter'=>'plain','tex_engine'=>'xelatex',
+	       'export_full_csv'=>[
+				  {-copy=>3,-question=>'deuxdeux',-score=>1},
+				  {-copy=>3,-question=>'fusion',-score=>2},
+				  ],
 	      )->default_process;
 
